/* global Services, Components */

'use strict';

this.EXPORTED_SYMBOLS = ['docObserver'];

Components.utils['import']('resource://gre/modules/Services.jsm');

var appName;

try { throw new Error; } catch (ex) {
	appName = ex.fileName.match(/:\/\/([^\/]+)/)[1];
}

let docObserver = {
	contentBaseURI: "chrome://" + appName + "/content/includes/",
	initContext: function(win, js) {
		let messager, Ci = Components.interfaces;

		try {
			// e10s
			messager = win
				.QueryInterface(Ci.nsIInterfaceRequestor)
				.getInterface(Ci.nsIWebNavigation)
				.QueryInterface(Ci.nsIDocShellTreeItem)
				.rootTreeItem
				.QueryInterface(Ci.nsIInterfaceRequestor)
				.getInterface(Ci.nsIContentFrameMessageManager);
		} catch (ex) {
			messager = win
				.QueryInterface(Ci.nsIInterfaceRequestor)
				.getInterface(Ci.nsIWebNavigation)
				.QueryInterface(Ci.nsIDocShell)
				.QueryInterface(Ci.nsIInterfaceRequestor)
				.getInterface(Ci.nsIContentFrameMessageManager);
		}

		if (js) {
			try {
				win = Components.utils.Sandbox([win], {
					sandboxPrototype: win,
					wantComponents: false,
					wantXHRConstructor: false
				});
			} catch (ex) {
				// This happens with media that are located under privileged addresses,
				// like on chrome: protocols, which we can freely pollute.
			}
		}

		win.sendAsyncMessage = messager.sendAsyncMessage;
		win.addMessageListener = messager[appName + '_addMessageListener'];
		win.removeMessageListener = messager[appName + '_removeMessageListener'];

		if (js) {
			let lss = Services.scriptloader.loadSubScript;
			lss(this.contentBaseURI + 'app.js', win);
			lss(this.contentBaseURI + js, win);
		}
	},
	mouseEnter: function(e) {
		this.removeEventListener(e.type, docObserver.mouseEnter, true);

		if (docObserver) {
			docObserver.initContext(this.ownerDocument.defaultView, 'opener.js');
		}
	},
	DOMReady: function(e) {
		if (e) {
			this.removeEventListener('DOMContentLoaded', docObserver.DOMReady, true);
		}

		if (docObserver) {
			this.documentElement.addEventListener('mouseenter', docObserver.mouseEnter, true);
		}
	},
	register: function() {
		Services.obs.addObserver(this, 'document-element-inserted', false);
	},
	unregister: function() {
		Services.obs.removeObserver(this, 'document-element-inserted');
	},
	observe: function(doc) {
		let win = doc.defaultView;

		if (!win) {
			return;
		}

		if (doc instanceof win.ImageDocument
			|| doc instanceof win.HTMLDocument
				&& doc.querySelector('body>video:only-child:not([src])')) {
			this.initContext(win, 'viewer.js');
			return;
		}

		let loc = win.location;

		if (loc.protocol === 'chrome:' && loc.host === appName) {
			this.initContext(win);
			return;
		}

		if (!/^(https?|f(tp|ile)):$/.test(loc.protocol)) {
			return;
		}

		doc.addEventListener('DOMContentLoaded', docObserver.DOMReady, true);
	}
};

docObserver.register();
