'use strict';

var cfg, Tabs, Port;

var app = {
	name: /*~APP_NAME~*/'Viewhance',
	version: /*~APP_VERSION~*/'0.9.7'
};


var Cc = Components.classes, Ci = Components.interfaces;
Components.utils['import']('resource://gre/modules/Services.jsm');

app.baseURI = 'chrome://' + app.name.toLowerCase()  + '/content/';

cfg = {
	PB: Services.prefs.getBranch('extensions.' + app.name + '.'),
	str: Cc['@mozilla.org/supports-string;1'].createInstance(Ci.nsISupportsString),
	getItem: function(k, fn) {
		try {
			fn(this.PB.getComplexValue(k, Ci.nsISupportsString).data);
		} catch (ex) {
			fn(null);
		}
	},
	setItem: function(k, v) {
		this.str.data = v;
		this.PB.setComplexValue(k, Ci.nsISupportsString, this.str);
	}
};

Tabs = {
	curWin: function() {
		return Services.wm.getMostRecentWindow('navigator:browser');
	},
	getSelected: function(cb) {
		var win = this.curWin();
		cb(win && win.gBrowser ? win.gBrowser.selectedTab : {});
	},
	create: function(p) {
		var win = this.curWin();

		if (win) {
			if (!/^[^:]{2,20}:/.test(p.url)) {
				p.url = app.baseURI + p.url;
			}

			win.gBrowser.loadOneTab(p.url, {inBackground: !p.active});
		}
	}
};

Port = {
	gmm: Cc['@mozilla.org/globalmessagemanager;1'].getService(Ci.nsIMessageListenerManager),
	frameScript: app.baseURI + 'frameScript.js',
	parse_msg: function(msg) {
		var listener_id = msg.data.listener_id;
		var messager = msg.target.messageManager;

		return {
			msg: msg.data.data,
			origin: msg.data.origin,
			postMessage: function(message) {
				messager.sendAsyncMessage(listener_id, JSON.stringify(message));
			}
		};
	},
	listen: function(fn) {
		if (typeof fn === 'function') {
			this.gmm.addMessageListener(app.name + ':background', fn);
			this.postMessage = fn;
		}
		else {
			this.gmm.removeMessageListener(app.name + ':background', this.postMessage);
			this.postMessage = null;
		}
	}
};

Port.gmm.loadFrameScript(Port.frameScript, true);

window.addEventListener('unload', function() {
	Port.gmm.removeMessageListener(app.name + ':background', Port.postMessage);
	Port.gmm.removeDelayedFrameScript(Port.frameScript);

	var URI = Port.frameScript.replace('Script.', 'Module.');
	var frameModule = {};
	Components.utils['import'](URI, frameModule);
	frameModule.docObserver.unregister();
	Components.utils.unload(URI);

	// close extension tabs
	var enumerator = Services.wm.getEnumerator('navigator:browser');
	var host = app.name.toLowerCase();
	var gBrowser, tabs, i, extURI;

	while (enumerator.hasMoreElements()) {
		gBrowser = enumerator.getNext().gBrowser;
		tabs = gBrowser.tabs;
		i = tabs.length;

		while (i--) {
			extURI = tabs[i].linkedBrowser.currentURI;

			if (extURI.scheme === 'chrome' && extURI.host === host) {
				gBrowser.removeTab(tabs[i]);
			}
		}
	}
});
